@@ -17,8 +17,11 @@ from .definition import (
     load_raw_definition,
     normalize_automation_definition,
     normalize_block_dictionaries,
+    normalize_data,
     normalize_data_dictionary,
     normalize_definition,
+    normalize_environment_variable_dictionaries,
+    normalize_file_dictionary,
     normalize_project_definition,
     normalize_result_definition,
     normalize_result_variable_dictionaries,
