@@ -1,14 +1,17 @@
-# TODO: Consider sanitize_json_values for result input, output, log, debug
+# TODO: Consider sanitize_json_value for result input, output, log, debug
 
+import json
 import re
 import requests
 import shlex
 import subprocess
+import time
 from collections import defaultdict
 from copy import deepcopy
 from functools import partial
 from invisibleroads_macros_disk import (
     TemporaryStorage, make_folder, make_random_folder)
+from io import StringIO
 from itertools import chain, product
 from mimetypes import guess_type
 from os import environ, getcwd
@@ -35,11 +38,14 @@ from .serialization import (
     SAVE_BY_EXTENSION_BY_VIEW)
 from ..constants import DEBUG_VARIABLE_DEFINITIONS, S
 from ..exceptions import (
+    CrossComputeConnectionError,
     CrossComputeDefinitionError,
     CrossComputeError,
     CrossComputeExecutionError,
     CrossComputeImplementationError,
     CrossComputeKeyboardInterrupt)
+from ..macros import (
+    sanitize_json_value)
 from ..symmetries import download
 
 
@@ -97,11 +103,12 @@ def run_tool(tool_definition, result_dictionary, script_command=None):
     run_script(script_command.format(
         **folder_by_key), script_folder, script_environment)
     for folder_name in 'output', 'log', 'debug':
-        if folder_name not in tool_definition:
-            continue
-        variable_definitions = tool_definition[folder_name]['variables']
+        variable_definitions = get_nested_value(
+            tool_definition, folder_name, 'variables', [])
         if folder_name == 'debug':
             variable_definitions += DEBUG_VARIABLE_DEFINITIONS
+        if not variable_definitions:
+            continue
         result_dictionary[folder_name] = {
             'variables': process_variable_folder(folder_by_name[
                 folder_name], variable_definitions)}
@@ -120,26 +127,38 @@ def run_worker(
         test_summary = run_tests(tool_definition)
         if not is_quiet:
             print(render_object(test_summary, as_json))
-    d = {'result count': 0}
-    try:
-        for event_name, event_dictionary in yield_echo(d, is_quiet, as_json):
-            if event_name == 'i' or d['ping count'] % 100 == 0:
-                d['result count'] += process_result_input_stream(
-                    script_command, is_quiet, as_json)
-    except CrossComputeKeyboardInterrupt:
-        pass
+    d = defaultdict(int)
+    while True:
+        try:
+            for [
+                event_name, event_dictionary,
+            ] in yield_echo(d, is_quiet, as_json):
+                if event_name == 'i' or d['ping count'] % 100 == 0:
+                    d['result count'] += process_result_input_stream(
+                        script_command, is_quiet, as_json)
+        except CrossComputeKeyboardInterrupt:
+            break
+        except (
+            CrossComputeConnectionError,
+            requests.exceptions.HTTPError,
+        ) as e:
+            print(e)
+            time.sleep(1)
+        except Exception:
+            print_exception(*exc_info())
+            time.sleep(1)
     if not is_quiet and not as_json:
         print('\n' + get_bash_configuration_text())
         print('cd ' + getcwd())
         print('crosscompute workers run')
-    return d
+    return dict(d)
 
 
 def run_script(script_command, script_folder, script_environment):
     script_arguments = shlex.split(script_command)
     debug_folder = script_environment.get('CROSSCOMPUTE_DEBUG_FOLDER', '')
-    stdout_file = open(join(debug_folder, 'stdout.log'), 'w+t')
-    stderr_file = open(join(debug_folder, 'stderr.log'), 'w+t')
+    stdout_file = open(join(debug_folder, 'stdout.txt'), 'w+t')
+    stderr_file = open(join(debug_folder, 'stderr.txt'), 'w+t')
     try:
         subprocess.run(
             script_arguments, env=script_environment, cwd=script_folder,
@@ -190,7 +209,7 @@ def run_tests(tool_definition):
 
 def render_result(tool_definition, result_dictionary):
     blocks = render_blocks(tool_definition, result_dictionary)
-    styles = result_dictionary.get('style', {}).get('rules', [])
+    styles = result_dictionary.get('print', {}).get('style', {}).get('rules', [])
     document_dictionary = {
         'blocks': blocks,
         'styles': styles,
@@ -333,7 +352,11 @@ def prepare_dataset(file_path, file_view, project_dictionaries):
     file_dictionary = dataset_dictionary['file']
     file_id = file_dictionary['id']
     file_url = file_dictionary['url']
-    requests.put(file_url, data=open(file_path, 'rb'))
+    file_object = open(file_path, 'rb')
+    if file_path.endswith('json'):
+        d = sanitize_json_value(json.load(file_object))
+        file_object = StringIO(json.dumps(d))
+    requests.put(file_url, data=file_object)
     fetch_resource('files', file_id, method='PATCH')
     return {'id': dataset_id, 'version': {'id': dataset_version_id}}
 
@@ -439,12 +462,19 @@ def process_variable_folder(folder, variable_definitions):
 
 def process_variable_dictionary(
         variable_dictionary, variable_definition, prepare_dataset):
-    variable_value = get_nested_value(variable_dictionary, 'data', 'value')
+    variable_view = variable_definition['view']
+    variable_data = variable_dictionary['data']
+    if 'value' not in variable_data:
+        return variable_dictionary
+    variable_value = variable_data['value']
     variable_value_length = len(repr(variable_value))
-    if variable_value_length < S['maximum_variable_value_length']:
+    if variable_view in [
+        'text',
+        'number',
+        'markdown',
+    ] and variable_value_length < S['maximum_variable_value_length']:
         return variable_dictionary
     variable_id = variable_definition['id']
-    variable_view = variable_definition['view']
     file_extension, save = list(SAVE_BY_EXTENSION_BY_VIEW[
         variable_view].items())[0]
     file_name = f'{variable_id}{file_extension}'
